import uuid

from datetime import date, datetime
from typing import TYPE_CHECKING

from sqlalchemy import Date, DateTime, ForeignKey, Unicode, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .account_base import AccountBase
    from .rate_contract_item import RateContractItem
    from .user import User


@reg.mapped_as_dataclass(unsafe_hash=True)
class RateContract:
    __tablename__ = "rate_contracts"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    date_: Mapped[date] = mapped_column("date", Date, nullable=False, index=True)
    vendor_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("accounts.id"), nullable=False)
    valid_from: Mapped[date | None] = mapped_column(Date(), nullable=True)
    valid_till: Mapped[date | None] = mapped_column(Date(), nullable=True)
    narration: Mapped[str] = mapped_column(Unicode, nullable=False)

    user_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("users.id"), nullable=False)
    creation_date: Mapped[datetime] = mapped_column(DateTime(), nullable=False)
    last_edit_date: Mapped[datetime] = mapped_column(DateTime(), nullable=False)

    user: Mapped["User"] = relationship("User")
    vendor: Mapped["AccountBase"] = relationship("AccountBase", back_populates="rate_contracts")
    items: Mapped[list["RateContractItem"]] = relationship("RateContractItem", back_populates="rate_contract")

    def __init__(
        self,
        date_: date,
        vendor_id: uuid.UUID,
        valid_from: date | None,
        valid_till: date | None,
        user_id: uuid.UUID,
        narration: str = "",
        creation_date: datetime | None = None,
        last_edit_date: datetime | None = None,
        id_: uuid.UUID | None = None,
    ):
        self.date_ = date_
        self.vendor_id = vendor_id
        self.valid_from = valid_from
        self.valid_till = valid_till
        self.narration = narration
        self.user_id = user_id
        self.creation_date = creation_date or datetime.utcnow()
        self.last_edit_date = last_edit_date or datetime.utcnow()
        if id_ is not None:
            self.id = id_
