from datetime import date, datetime

from brewman.schemas import to_camel
from brewman.schemas.account import AccountLink
from brewman.schemas.attendance_type import AttendanceType
from pydantic import (
    BaseModel,
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)


class EmployeeAttendanceItem(BaseModel):
    date_: date
    attendance_type: AttendanceType
    prints: str
    hours_worked: str
    full_day: bool | None = None
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")


class EmployeeAttendance(BaseModel):
    start_date: date | None = None
    finish_date: date | None = None
    employee: AccountLink | None = None
    body: list[EmployeeAttendanceItem]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("start_date", mode="before")
    @classmethod
    def parse_start_date(cls, value: date | str | None) -> date | None:
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("start_date")
    def serialize_start_date(self, value: date, info: FieldSerializationInfo) -> str | None:
        return None if value is None else value.strftime("%d-%b-%Y")

    @field_validator("finish_date", mode="before")
    @classmethod
    def parse_finish_date(cls, value: date | str | None) -> date | None:
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("finish_date")
    def serialize_finish_date(self, value: date, info: FieldSerializationInfo) -> str | None:
        return None if value is None else value.strftime("%d-%b-%Y")
