import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';

import { AttendanceResolver } from './attendance-resolver.service';
import { AttendanceTypeResolver } from './attendance-type-resolver.service';
import { AttendanceComponent } from './attendance.component';

const attendanceRoutes: Routes = [
  {
    path: '',
    component: AttendanceComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Attendance',
    },
    resolve: {
      info: (route: ActivatedRouteSnapshot) => inject(AttendanceResolver).resolve(route),
      attendanceTypes: () => inject(AttendanceTypeResolver).resolve(),
    },
    runGuardsAndResolvers: 'always',
  },
  {
    path: ':date',
    component: AttendanceComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Attendance',
    },
    resolve: {
      info: (route: ActivatedRouteSnapshot) => inject(AttendanceResolver).resolve(route),
      attendanceTypes: () => inject(AttendanceTypeResolver).resolve(),
    },
    runGuardsAndResolvers: 'always',
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(attendanceRoutes)],
  exports: [RouterModule],
  providers: [AttendanceResolver],
})
export class AttendanceRoutingModule {}
