import { Component, OnInit } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { CashFlow } from './cash-flow';
import { CashFlowDataSource } from './cash-flow-datasource';

@Component({
  selector: 'app-cash-flow',
  templateUrl: './cash-flow.component.html',
  styleUrls: ['./cash-flow.component.css'],
})
export class CashFlowComponent implements OnInit {
  info: CashFlow = new CashFlow();
  dataSource: CashFlowDataSource = new CashFlowDataSource(CashFlow.Data(this.info));
  form: FormGroup<{
    startDate: FormControl<Date>;
    finishDate: FormControl<Date>;
  }>;

  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['name', 'amount'];

  constructor(
    private route: ActivatedRoute,
    private router: Router,
  ) {
    this.form = new FormGroup({
      startDate: new FormControl(new Date(), { nonNullable: true }),
      finishDate: new FormControl(new Date(), { nonNullable: true }),
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { info: CashFlow };

      this.info = data.info;
      this.form.setValue({
        startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
        finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate(),
      });
      this.dataSource = new CashFlowDataSource(CashFlow.Data(this.info));
    });
  }

  show() {
    const info = this.getInfo();
    this.router.navigate(['cash-flow'], {
      queryParams: {
        startDate: info.startDate,
        finishDate: info.finishDate,
      },
    });
  }

  getInfo(): CashFlow {
    const formModel = this.form.value;

    return new CashFlow({
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY'),
    });
  }
}
