import { CommonModule } from '@angular/common';
import { NgModule, inject } from '@angular/core';
import { ActivatedRouteSnapshot, RouterModule, RouterStateSnapshot, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';
import { ProductGroupListResolver } from '../product-group/product-group-list-resolver.service';

import { ProductDetailComponent } from './product-detail/product-detail.component';
import { ProductListComponent } from './product-list/product-list.component';
import { ProductListResolver } from './product-list-resolver.service';
import { ProductResolver } from './product-resolver.service';

const productRoutes: Routes = [
  {
    path: '',
    component: ProductListComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Products',
    },
    resolve: {
      list: () => inject(ProductListResolver).resolve(),
    },
  },
  {
    path: 'new',
    component: ProductDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Products',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(ProductResolver).resolve(route),
      productGroups: () => inject(ProductGroupListResolver).resolve(),
    },
  },
  {
    path: ':id',
    component: ProductDetailComponent,
    canActivate: [
      (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => inject(AuthGuard).canActivate(route, state),
    ],
    data: {
      permission: 'Products',
    },
    resolve: {
      item: (route: ActivatedRouteSnapshot) => inject(ProductResolver).resolve(route),
      productGroups: () => inject(ProductGroupListResolver).resolve(),
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(productRoutes)],
  exports: [RouterModule],
  providers: [ProductListResolver, ProductResolver],
})
export class ProductRoutingModule {}
