import { Component, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { Purchases } from './purchases';
import { PurchasesDataSource } from './purchases-datasource';
import { PurchasesItem } from './purchases-item';

@Component({
  selector: 'app-purchases',
  templateUrl: './purchases.component.html',
  styleUrls: ['./purchases.component.css'],
})
export class PurchasesComponent implements OnInit {
  @ViewChild(MatPaginator, { static: true }) paginator?: MatPaginator;
  @ViewChild(MatSort, { static: true }) sort?: MatSort;
  info: Purchases = new Purchases();
  dataSource: PurchasesDataSource = new PurchasesDataSource(this.info.body);
  form: FormGroup<{
    startDate: FormControl<Date>;
    finishDate: FormControl<Date>;
  }>;

  selectedRowId = '';
  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['product', 'quantity', 'rate', 'amount'];

  constructor(
    private route: ActivatedRoute,
    private router: Router,
  ) {
    this.form = new FormGroup({
      startDate: new FormControl(new Date(), { nonNullable: true }),
      finishDate: new FormControl(new Date(), { nonNullable: true }),
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { info: Purchases };

      this.info = data.info;
      this.form.setValue({
        startDate: moment(this.info.startDate, 'DD-MMM-YYYY').toDate(),
        finishDate: moment(this.info.finishDate, 'DD-MMM-YYYY').toDate(),
      });
      this.dataSource = new PurchasesDataSource(this.info.body, this.paginator, this.sort);
    });
  }

  show() {
    const l = this.prepareSubmit();
    this.router.navigate(['purchases'], {
      queryParams: {
        startDate: l.startDate,
        finishDate: l.finishDate,
      },
    });
  }

  prepareSubmit(): Purchases {
    const formModel = this.form.value;

    return {
      startDate: moment(formModel.startDate).format('DD-MMM-YYYY'),
      finishDate: moment(formModel.finishDate).format('DD-MMM-YYYY'),
      body: [],
      footer: new PurchasesItem(),
    };
  }
}
